# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from verity471.models.activity import Activity
from verity471.models.activity_location import ActivityLocation
from verity471.models.classification import Classification
from verity471.models.cve_source import CveSource
from verity471.models.cvss import Cvss
from verity471.models.exploit_status import ExploitStatus
from verity471.models.interest_level import InterestLevel
from verity471.models.link1 import Link1
from verity471.models.links import Links
from verity471.models.patch_status import PatchStatus
from verity471.models.poc import Poc
from verity471.models.risk_level import RiskLevel
from verity471.models.vulnerability_status import VulnerabilityStatus
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

if TYPE_CHECKING:
    # Only for type checkers; does not run at runtime.
    from verity471.verity_stix import STIXMapperSettings


class VulnerabilitiesReportDetailsResponse(BaseModel):
    """
    Minimal schema for a vulnerability report response with keys, types, and descriptions
    """ # noqa: E501
    activity: Activity
    activity_location: Optional[List[ActivityLocation]] = Field(default=None, description="List of locations where activity related to the vulnerability has been observed")
    aliases: Optional[List[StrictStr]] = Field(default=None, description="List of alternative names or identifiers for the vulnerability")
    body: Optional[StrictStr] = Field(default=None, description="HTML formatted summary of the vulnerability")
    classification: Classification
    counter_measure_links: Optional[List[Link1]] = Field(default=None, description="List of titled URLs to Counter Measures related to the vulnerability")
    counter_measures_html: Optional[StrictStr] = Field(default=None, description="HTML formatted summary of countermeasures for the vulnerability")
    creation_ts: StrictStr = Field(description="Timestamp when the report was published first time")
    cve_type: StrictStr = Field(description="Type of CVE, for example: Buffer overflow, Privilege escalation, Memory corruption, etc")
    cvss: Optional[List[Cvss]] = Field(default=None, description="List of CVSS scores associated with the vulnerability")
    exploit_status: Optional[List[ExploitStatus]] = Field(default=None, description="List of exploitation status details for the vulnerability")
    id: StrictStr = Field(description="Unique identifier of the report")
    interest_level: Optional[List[InterestLevel]] = Field(default=None, description="List of levels of interest related to the vulnerability")
    last_updated_ts: StrictStr = Field(description="Timestamp of last report update")
    links: Links
    name: StrictStr = Field(description="Name of the vulnerability")
    patch_links: Optional[List[Link1]] = Field(default=None, description="List of titled URLs to Patches related to the vulnerability")
    patch_status: Optional[PatchStatus] = None
    poc: Optional[Poc] = None
    poc_links: Optional[List[Link1]] = Field(default=None, description="List of titled URLs to Proofs of Concept related to the vulnerability")
    product_name: StrictStr = Field(description="Name of the product of the affected software")
    released_ts: StrictStr = Field(description="Timestamp when the report was published last time")
    risk_level: RiskLevel
    sort_priority: Optional[StrictInt] = Field(default=None, description="Priority used for sorting vulnerability reports")
    sources: Optional[List[CveSource]] = Field(default=None, description="List of sources referenced in the report")
    status: VulnerabilityStatus
    type: StrictStr = Field(description="Type of the report")
    underground_activity_summary_html: Optional[StrictStr] = Field(default=None, description="HTML formatted summary of underground activity related to the vulnerability")
    vendor_name: StrictStr = Field(description="Name of the vendor of the affected software")
    __properties: ClassVar[List[str]] = ["activity", "activity_location", "aliases", "body", "classification", "counter_measure_links", "counter_measures_html", "creation_ts", "cve_type", "cvss", "exploit_status", "id", "interest_level", "last_updated_ts", "links", "name", "patch_links", "patch_status", "poc", "poc_links", "product_name", "released_ts", "risk_level", "sort_priority", "sources", "status", "type", "underground_activity_summary_html", "vendor_name"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_stix(self, settings: Optional["STIXMapperSettings"] = None):
        try:
            from verity471.verity_stix.mappers.common import StixMapper
        except ImportError as e:
            raise ImportError(
                "STIX support is an optional feature. "
                "Install with: pip install 'verity471[stix]'"
            ) from e
        stix_mapper = StixMapper(settings)
        return stix_mapper.map(self.to_dict())

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of VulnerabilitiesReportDetailsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of activity
        if self.activity:
            _dict['activity'] = self.activity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of classification
        if self.classification:
            _dict['classification'] = self.classification.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in counter_measure_links (list)
        _items = []
        if self.counter_measure_links:
            for _item_counter_measure_links in self.counter_measure_links:
                if _item_counter_measure_links:
                    _items.append(_item_counter_measure_links.to_dict())
            _dict['counter_measure_links'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in cvss (list)
        _items = []
        if self.cvss:
            for _item_cvss in self.cvss:
                if _item_cvss:
                    _items.append(_item_cvss.to_dict())
            _dict['cvss'] = _items
        # override the default output from pydantic by calling `to_dict()` of links
        if self.links:
            _dict['links'] = self.links.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in patch_links (list)
        _items = []
        if self.patch_links:
            for _item_patch_links in self.patch_links:
                if _item_patch_links:
                    _items.append(_item_patch_links.to_dict())
            _dict['patch_links'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in poc_links (list)
        _items = []
        if self.poc_links:
            for _item_poc_links in self.poc_links:
                if _item_poc_links:
                    _items.append(_item_poc_links.to_dict())
            _dict['poc_links'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item_sources in self.sources:
                if _item_sources:
                    _items.append(_item_sources.to_dict())
            _dict['sources'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of VulnerabilitiesReportDetailsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "activity": Activity.from_dict(obj["activity"]) if obj.get("activity") is not None else None,
            "activity_location": obj.get("activity_location"),
            "aliases": obj.get("aliases"),
            "body": obj.get("body"),
            "classification": Classification.from_dict(obj["classification"]) if obj.get("classification") is not None else None,
            "counter_measure_links": [Link1.from_dict(_item) for _item in obj["counter_measure_links"]] if obj.get("counter_measure_links") is not None else None,
            "counter_measures_html": obj.get("counter_measures_html"),
            "creation_ts": obj.get("creation_ts"),
            "cve_type": obj.get("cve_type"),
            "cvss": [Cvss.from_dict(_item) for _item in obj["cvss"]] if obj.get("cvss") is not None else None,
            "exploit_status": obj.get("exploit_status"),
            "id": obj.get("id"),
            "interest_level": obj.get("interest_level"),
            "last_updated_ts": obj.get("last_updated_ts"),
            "links": Links.from_dict(obj["links"]) if obj.get("links") is not None else None,
            "name": obj.get("name"),
            "patch_links": [Link1.from_dict(_item) for _item in obj["patch_links"]] if obj.get("patch_links") is not None else None,
            "patch_status": obj.get("patch_status"),
            "poc": obj.get("poc"),
            "poc_links": [Link1.from_dict(_item) for _item in obj["poc_links"]] if obj.get("poc_links") is not None else None,
            "product_name": obj.get("product_name"),
            "released_ts": obj.get("released_ts"),
            "risk_level": obj.get("risk_level"),
            "sort_priority": obj.get("sort_priority"),
            "sources": [CveSource.from_dict(_item) for _item in obj["sources"]] if obj.get("sources") is not None else None,
            "status": obj.get("status"),
            "type": obj.get("type"),
            "underground_activity_summary_html": obj.get("underground_activity_summary_html"),
            "vendor_name": obj.get("vendor_name")
        })
        return _obj





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from verity471.models.geopol_report_details_response import GeopolReportDetailsResponse
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

if TYPE_CHECKING:
    # Only for type checkers; does not run at runtime.
    from verity471.verity_stix import STIXMapperSettings


class GeopolReportsResponseStream(BaseModel):
    """
    GeopolReportsResponseStream
    """ # noqa: E501
    adversary_profile_count: StrictInt = Field(description="Count of Adversary Profile reports")
    count: StrictInt = Field(description="Number of total Geopol reports matching the query")
    cursor_next: Optional[StrictStr] = Field(default=None, description="Stream position identifier to continue scrolling from")
    intelligence_bulletin_count: StrictInt = Field(description="Count of Intelligence Bulletin reports")
    intelligence_estimate_count: StrictInt = Field(description="Count of Intelligence Estimate reports")
    intelligence_summary_count: StrictInt = Field(description="Count of Intelligence Summary reports")
    reports: Optional[List[GeopolReportDetailsResponse]] = Field(default=None, description="List of Geopol reports in the current batch")
    sigact_count: StrictInt = Field(description="Count of Significant Activity Report reports")
    spot_count: StrictInt = Field(description="Count of Geopol Spot Report reports")
    tension_point_profile_count: StrictInt = Field(description="Count of Tension Point Profile reports")
    threat_brief_count: StrictInt = Field(description="Count of Threat Brief reports")
    __properties: ClassVar[List[str]] = ["adversary_profile_count", "count", "cursor_next", "intelligence_bulletin_count", "intelligence_estimate_count", "intelligence_summary_count", "reports", "sigact_count", "spot_count", "tension_point_profile_count", "threat_brief_count"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_stix(self, settings: Optional["STIXMapperSettings"] = None):
        try:
            from verity471.verity_stix.mappers.common import StixMapper
        except ImportError as e:
            raise ImportError(
                "STIX support is an optional feature. "
                "Install with: pip install 'verity471[stix]'"
            ) from e
        stix_mapper = StixMapper(settings)
        return stix_mapper.map(self.to_dict())

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GeopolReportsResponseStream from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in reports (list)
        _items = []
        if self.reports:
            for _item_reports in self.reports:
                if _item_reports:
                    _items.append(_item_reports.to_dict())
            _dict['reports'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GeopolReportsResponseStream from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "adversary_profile_count": obj.get("adversary_profile_count"),
            "count": obj.get("count"),
            "cursor_next": obj.get("cursor_next"),
            "intelligence_bulletin_count": obj.get("intelligence_bulletin_count"),
            "intelligence_estimate_count": obj.get("intelligence_estimate_count"),
            "intelligence_summary_count": obj.get("intelligence_summary_count"),
            "reports": [GeopolReportDetailsResponse.from_dict(_item) for _item in obj["reports"]] if obj.get("reports") is not None else None,
            "sigact_count": obj.get("sigact_count"),
            "spot_count": obj.get("spot_count"),
            "tension_point_profile_count": obj.get("tension_point_profile_count"),
            "threat_brief_count": obj.get("threat_brief_count")
        })
        return _obj





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from verity471.models.activity import Activity
from verity471.models.classification import Classification
from verity471.models.indicator_data import IndicatorData
from verity471.models.kill_chain_phase import KillChainPhase
from verity471.models.revocation import Revocation
from verity471.models.threat import Threat
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

if TYPE_CHECKING:
    # Only for type checkers; does not run at runtime.
    from verity471.verity_stix import STIXMapperSettings


class IntegrationsIndicator(BaseModel):
    """
    Indicator data
    """ # noqa: E501
    activity: Optional[Activity] = Field(default=None, description="Activity information related to the entity")
    classification: Optional[Classification] = Field(default=None, description="Classification information for the entity")
    confidence: Optional[StrictInt] = Field(default=None, description="Confidence score for the indicator (0-100)")
    data: Optional[IndicatorData] = Field(default=None, description="Indicator data containing specific details")
    description: Optional[StrictStr] = Field(default=None, description="Descriptive information about the indicator")
    expiration_ts: Optional[datetime] = Field(default=None, description="Timestamp when the indicator expires")
    id: Optional[StrictStr] = Field(default=None, description="Unique identifier of the entity")
    kill_chain_phases: Optional[List[KillChainPhase]] = Field(default=None, description="List of kill chain phases associated with the entity")
    pattern: Optional[StrictStr] = Field(default=None, description="Pattern string for the indicator")
    pattern_type: Optional[StrictStr] = Field(default=None, description="Type of pattern used for the indicator")
    pattern_version: Optional[StrictStr] = Field(default=None, description="Version of the pattern specification")
    revocation: Optional[Revocation] = Field(default=None, description="Revocation information for the indicator, if applicable")
    threat: Optional[Threat] = Field(default=None, description="Threat information associated with the entity")
    type: Optional[StrictStr] = Field(default=None, description="Type of indicator")
    __properties: ClassVar[List[str]] = ["activity", "classification", "confidence", "data", "description", "expiration_ts", "id", "kill_chain_phases", "pattern", "pattern_type", "pattern_version", "revocation", "threat", "type"]

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['domain', 'email', 'file', 'ipv4', 'url', 'yara']):
            raise ValueError("must be one of enum values ('domain', 'email', 'file', 'ipv4', 'url', 'yara')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_stix(self, settings: Optional["STIXMapperSettings"] = None):
        try:
            from verity471.verity_stix.mappers.common import StixMapper
        except ImportError as e:
            raise ImportError(
                "STIX support is an optional feature. "
                "Install with: pip install 'verity471[stix]'"
            ) from e
        stix_mapper = StixMapper(settings)
        return stix_mapper.map(self.to_dict())

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of IntegrationsIndicator from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of activity
        if self.activity:
            _dict['activity'] = self.activity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of classification
        if self.classification:
            _dict['classification'] = self.classification.to_dict()
        # override the default output from pydantic by calling `to_dict()` of data
        if self.data:
            _dict['data'] = self.data.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in kill_chain_phases (list)
        _items = []
        if self.kill_chain_phases:
            for _item_kill_chain_phases in self.kill_chain_phases:
                if _item_kill_chain_phases:
                    _items.append(_item_kill_chain_phases.to_dict())
            _dict['kill_chain_phases'] = _items
        # override the default output from pydantic by calling `to_dict()` of revocation
        if self.revocation:
            _dict['revocation'] = self.revocation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of threat
        if self.threat:
            _dict['threat'] = self.threat.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of IntegrationsIndicator from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "activity": Activity.from_dict(obj["activity"]) if obj.get("activity") is not None else None,
            "classification": Classification.from_dict(obj["classification"]) if obj.get("classification") is not None else None,
            "confidence": obj.get("confidence"),
            "data": IndicatorData.from_dict(obj["data"]) if obj.get("data") is not None else None,
            "description": obj.get("description"),
            "expiration_ts": obj.get("expiration_ts"),
            "id": obj.get("id"),
            "kill_chain_phases": [KillChainPhase.from_dict(_item) for _item in obj["kill_chain_phases"]] if obj.get("kill_chain_phases") is not None else None,
            "pattern": obj.get("pattern"),
            "pattern_type": obj.get("pattern_type"),
            "pattern_version": obj.get("pattern_version"),
            "revocation": Revocation.from_dict(obj["revocation"]) if obj.get("revocation") is not None else None,
            "threat": Threat.from_dict(obj["threat"]) if obj.get("threat") is not None else None,
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr, field_validator
from typing import Optional
from typing_extensions import Annotated
from verity471.models.indicators_stream import IndicatorsStream
from verity471.models.integrations_indicator import IntegrationsIndicator

from verity471.api_client import ApiClient, RequestSerialized
from verity471.api_response import ApiResponse
from verity471.rest import RESTResponseType


class IndicatorsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_indicator_by_id(
        self,
        id: Annotated[StrictStr, Field(description="Id of the indicator")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> IntegrationsIndicator:
        """Get indicator by id


        :param id: Id of the indicator (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicator_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IntegrationsIndicator",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_indicator_by_id_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="Id of the indicator")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[IntegrationsIndicator]:
        """Get indicator by id


        :param id: Id of the indicator (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicator_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IntegrationsIndicator",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_indicator_by_id_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="Id of the indicator")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get indicator by id


        :param id: Id of the indicator (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicator_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IntegrationsIndicator",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_indicator_by_id_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/indicators/v1/indicators/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_indicators_stream(
        self,
        type: Annotated[Optional[StrictStr], Field(description="Search indicators by type")] = None,
        threat_type: Annotated[Optional[StrictStr], Field(description="Search indicators by threat type")] = None,
        confidence: Annotated[Optional[StrictStr], Field(description="Search indicators by confidence")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        text_filter: Annotated[Optional[StrictStr], Field(description="Free text indicator search (all fields included)")] = None,
        malware_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware id")] = None,
        malware_family_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family id")] = None,
        malware_family_name: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list")] = None,
        size: Annotated[Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]], Field(description="The size of the page to be returned. Max size: `1000`")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`")] = None,
        until: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> IndicatorsStream:
        """Stream indicators using a cursor

        Indicators sorted in ascending order of processed time.

        :param type: Search indicators by type
        :type type: str
        :param threat_type: Search indicators by threat type
        :type threat_type: str
        :param confidence: Search indicators by confidence
        :type confidence: str
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param text_filter: Free text indicator search (all fields included)
        :type text_filter: str
        :param malware_id: Search indicators by malware id
        :type malware_id: str
        :param malware_family_id: Search indicators by malware family id
        :type malware_family_id: str
        :param malware_family_name: Search indicators by malware family
        :type malware_family_name: str
        :param girs: Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list
        :type girs: str
        :param size: The size of the page to be returned. Max size: `1000`
        :type size: int
        :param var_from: Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`
        :type var_from: int
        :param until: Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`
        :type until: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicators_stream_serialize(
            type=type,
            threat_type=threat_type,
            confidence=confidence,
            cursor=cursor,
            text_filter=text_filter,
            malware_id=malware_id,
            malware_family_id=malware_family_id,
            malware_family_name=malware_family_name,
            girs=girs,
            size=size,
            var_from=var_from,
            until=until,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IndicatorsStream",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_indicators_stream_with_http_info(
        self,
        type: Annotated[Optional[StrictStr], Field(description="Search indicators by type")] = None,
        threat_type: Annotated[Optional[StrictStr], Field(description="Search indicators by threat type")] = None,
        confidence: Annotated[Optional[StrictStr], Field(description="Search indicators by confidence")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        text_filter: Annotated[Optional[StrictStr], Field(description="Free text indicator search (all fields included)")] = None,
        malware_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware id")] = None,
        malware_family_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family id")] = None,
        malware_family_name: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list")] = None,
        size: Annotated[Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]], Field(description="The size of the page to be returned. Max size: `1000`")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`")] = None,
        until: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[IndicatorsStream]:
        """Stream indicators using a cursor

        Indicators sorted in ascending order of processed time.

        :param type: Search indicators by type
        :type type: str
        :param threat_type: Search indicators by threat type
        :type threat_type: str
        :param confidence: Search indicators by confidence
        :type confidence: str
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param text_filter: Free text indicator search (all fields included)
        :type text_filter: str
        :param malware_id: Search indicators by malware id
        :type malware_id: str
        :param malware_family_id: Search indicators by malware family id
        :type malware_family_id: str
        :param malware_family_name: Search indicators by malware family
        :type malware_family_name: str
        :param girs: Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list
        :type girs: str
        :param size: The size of the page to be returned. Max size: `1000`
        :type size: int
        :param var_from: Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`
        :type var_from: int
        :param until: Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`
        :type until: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicators_stream_serialize(
            type=type,
            threat_type=threat_type,
            confidence=confidence,
            cursor=cursor,
            text_filter=text_filter,
            malware_id=malware_id,
            malware_family_id=malware_family_id,
            malware_family_name=malware_family_name,
            girs=girs,
            size=size,
            var_from=var_from,
            until=until,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IndicatorsStream",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_indicators_stream_without_preload_content(
        self,
        type: Annotated[Optional[StrictStr], Field(description="Search indicators by type")] = None,
        threat_type: Annotated[Optional[StrictStr], Field(description="Search indicators by threat type")] = None,
        confidence: Annotated[Optional[StrictStr], Field(description="Search indicators by confidence")] = None,
        cursor: Annotated[Optional[StrictStr], Field(description="Continue scrolling from cursor")] = None,
        text_filter: Annotated[Optional[StrictStr], Field(description="Free text indicator search (all fields included)")] = None,
        malware_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware id")] = None,
        malware_family_id: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family id")] = None,
        malware_family_name: Annotated[Optional[StrictStr], Field(description="Search indicators by malware family")] = None,
        girs: Annotated[Optional[StrictStr], Field(description="Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list")] = None,
        size: Annotated[Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]], Field(description="The size of the page to be returned. Max size: `1000`")] = None,
        var_from: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`")] = None,
        until: Annotated[Optional[StrictInt], Field(description="Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Stream indicators using a cursor

        Indicators sorted in ascending order of processed time.

        :param type: Search indicators by type
        :type type: str
        :param threat_type: Search indicators by threat type
        :type threat_type: str
        :param confidence: Search indicators by confidence
        :type confidence: str
        :param cursor: Continue scrolling from cursor
        :type cursor: str
        :param text_filter: Free text indicator search (all fields included)
        :type text_filter: str
        :param malware_id: Search indicators by malware id
        :type malware_id: str
        :param malware_family_id: Search indicators by malware family id
        :type malware_family_id: str
        :param malware_family_name: Search indicators by malware family
        :type malware_family_name: str
        :param girs: Filter result by custom GIRs (General intel requirements), `my_girs` or `company_pirs`. Using multiple value will return result based on the aggregated GIR list
        :type girs: str
        :param size: The size of the page to be returned. Max size: `1000`
        :type size: int
        :param var_from: Long unix timestamp in milliseconds. Search data starting from given `activity.first_seen_ts` and `activity.last_seen_ts` (including). Example - `1627776000000`
        :type var_from: int
        :param until: Long unix timestamp in milliseconds. Search data ending before given `activity.first_seen_ts` and `activity.last_seen_ts` (excluding). Example - `1627776000000`
        :type until: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_indicators_stream_serialize(
            type=type,
            threat_type=threat_type,
            confidence=confidence,
            cursor=cursor,
            text_filter=text_filter,
            malware_id=malware_id,
            malware_family_id=malware_family_id,
            malware_family_name=malware_family_name,
            girs=girs,
            size=size,
            var_from=var_from,
            until=until,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "IndicatorsStream",
            '400': "ErrorResponse",
            '401': "Unauthorized",
            '403': "Forbidden",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_indicators_stream_serialize(
        self,
        type,
        threat_type,
        confidence,
        cursor,
        text_filter,
        malware_id,
        malware_family_id,
        malware_family_name,
        girs,
        size,
        var_from,
        until,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if type is not None:
            
            _query_params.append(('type', type))
            
        if threat_type is not None:
            
            _query_params.append(('threat_type', threat_type))
            
        if confidence is not None:
            
            _query_params.append(('confidence', confidence))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        if text_filter is not None:
            
            _query_params.append(('text_filter', text_filter))
            
        if malware_id is not None:
            
            _query_params.append(('malware_id', malware_id))
            
        if malware_family_id is not None:
            
            _query_params.append(('malware_family_id', malware_family_id))
            
        if malware_family_name is not None:
            
            _query_params.append(('malware_family_name', malware_family_name))
            
        if girs is not None:
            
            _query_params.append(('girs', girs))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if until is not None:
            
            _query_params.append(('until', until))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/integrations/indicators/v1/indicators/stream',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





import json

import pytest

from verity471.verity_stix import STIXMapperSettings
from verity471.verity_stix.exceptions import StixMapperNotFound
from verity471.verity_stix.mappers import ReportMapper
from verity471.verity_stix.mappers.common import StixMapper
from verity471.verity_stix.mappers.entities import EntitiesMapper
from verity471.verity_stix.mappers.indicators import IndicatorsMapper
from verity471.verity_stix.mappers.reports import ReportType

from .conftest import PREFIX, read_fixture
from unittest.mock import MagicMock



def strip_random_values(bundle: dict) -> dict:
    bundle["id"] = None
    for i1, o in enumerate(bundle["objects"]):
        bundle["objects"][i1]["created"] = None
        bundle["objects"][i1]["modified"] = None
    return bundle


@pytest.mark.parametrize(
    "in_fixture, out_fixture",
    [
        pytest.param("indicators_ipv4_input.json", "indicators_ipv4_stix.json", id="indicators-ipv4"),
        pytest.param("indicators_file_input.json", "indicators_file_stix.json", id="indicators-file"),
        pytest.param("indicators_url_input.json", "indicators_url_stix.json", id="indicators-url"),
        pytest.param("indicators_yara_input.json", "indicators_yara_stix.json", id="indicators-yara"),
        pytest.param("indicators_domain_input.json", "indicators_domain_stix.json", id="indicators-domain"),
        pytest.param("indicators_email_input.json", "indicators_email_stix.json", id="indicators-email"),
        pytest.param("cves_input.json", "cves_stix.json", id="cves"),
        pytest.param("report_breach_alert_input.json", "report_breach_alert_stix.json", id="report_breach_alert"),
        pytest.param("report_fintel_input.json", "report_fintel_stix.json", id="report_fintel"),
        pytest.param("report_inforep_input.json", "report_inforep_stix.json", id="report_inforep"),
        pytest.param("report_spot_input.json", "report_spot_stix.json", id="report_spot"),
        pytest.param("report_malware_input.json", "report_malware_stix.json", id="report_malware"),
        pytest.param("report_geopol_input.json", "report_geopol_stix.json", id="report_geopol"),
    ]
)
@pytest.mark.parametrize(
    "by_id", [
        pytest.param(True, id="by-id"),
        pytest.param(False, id="stream"),
    ]
)
def test_stix_mappers(in_fixture, out_fixture, by_id):
    api_response = read_fixture(f'{PREFIX}/fixtures/{in_fixture}')
    if by_id:
        for value in api_response.values():
            if isinstance(value, list) and value and isinstance(value[0], dict):
                api_response = value[0]
    expected_result = read_fixture(f'{PREFIX}/fixtures/{out_fixture}')

    mapper = StixMapper(STIXMapperSettings(
        client=MagicMock(name="client"), api_client=MagicMock(name="api_client"), report_full_content=False
    ))
    result = mapper.map(api_response)
    actual = json.loads(result.serialize())

    actual_norm = strip_random_values(actual)
    expected_norm = strip_random_values(expected_result)
    assert expected_norm == actual_norm


fragment_truncated = {"is_truncated": True}
fragment_attachments = {"attachments": [{"mime_type" : "image/png", "url" : "https://api.intel471.cloud/intel-report/v1/reports/.../attachments/4b76f7665f5331a91531a1ae0dd27e9755b1eef41288ab3f3feec0c6fd7bfbae"}]}


@pytest.mark.parametrize(
    "report_full_content, stream_response_fragment, should_fetch",
    [
        pytest.param(True, fragment_truncated, True, id="truncated-1"),
        pytest.param(False, fragment_truncated, False, id="truncated-2"),
        pytest.param(True, {}, False, id="truncated-3"),
        pytest.param(True, fragment_attachments, True, id="attachments-1"),
        pytest.param(False, fragment_attachments, False, id="attachments-2"),
    ]
)
def test_report_mapper_fetches_full_report_on_demand(report_full_content, stream_response_fragment, should_fetch):
    api_cls = "ReportsApi"
    by_id_method_name = "get_reports_fintel_id"
    by_id_api_response = {
        "type": "fintel",
        "title": "BYID",
        "creation_ts": "2025-07-16T13:28:28Z",
        "id": "report--fc000675-a3ec-49b8-92a0-a0247f5e969e"
        }

    by_id_api_response_mock = MagicMock(name="By ID API response")
    by_id_api_response_mock.to_dict.return_value = by_id_api_response
    api_instance_mock = MagicMock(name="API instance")
    getattr(api_instance_mock, by_id_method_name).return_value = by_id_api_response_mock
    mock_api_client = MagicMock(name="mock_api_client")
    mock_verity471 = MagicMock(name="verity471")
    getattr(mock_verity471, api_cls).return_value = api_instance_mock

    mapper = ReportMapper(STIXMapperSettings(mock_verity471, mock_api_client, report_full_content=report_full_content))
    stream_api_response_report = {
        "type": "fintel",
        "title": "STREAM",
        "creation_ts": "2025-07-16T13:28:28Z",
        "id": "report--fc000675-a3ec-49b8-92a0-a0247f5e969e",
    }
    stream_api_response_report.update(stream_response_fragment)
    stream_api_response = {"count": "1", "reports": [stream_api_response_report]}
    bundle = mapper.map(stream_api_response)
    report = bundle.objects[-1]
    expected_name = "BYID" if should_fetch else "STREAM"
    assert report.name == expected_name


@pytest.mark.parametrize("source,expected_values", (
        # /iocs endpoint
        ({"type": "MaliciousURL", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "URL", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "MaliciousDomain", "value": "acme.com"}, {"type": "domain-name", "value": "acme.com"}),
        ({"type": "IPAddress", "value": "10.0.0.1"}, {"type": "ipv4-addr", "value": "10.0.0.1"}),
        ({"type": "IPv4Prefix", "value": "10.0.0.1/24"}, {"type": "ipv4-addr", "value": "10.0.0.1/24"}),
        ({"type": "IPv6Prefix", "value": "2a09:7c44::/32"}, {"type": "ipv6-addr", "value": "2a09:7c44::/32"}),
        ({"type": "AutonomousSystem", "value": "AS123456"}, {"type": "autonomous-system", "number": 123456}),
        ({"type": "MD5", "value": "acb9cf2560602aa023e9933b959974d1"}, {"type": "file", "hashes": {"MD5": "acb9cf2560602aa023e9933b959974d1"}}),
        ({"type": "SHA1", "value": "ae9de44e5f23758ffb6f4fa28065c6122c2e4467"}, {"type": "file", "hashes": {"SHA-1": "ae9de44e5f23758ffb6f4fa28065c6122c2e4467"}}),
        ({"type": "SHA256", "value": "890a0bfab48d0b93da8f7617b2e65621e8d7f8c93a854fa8596232d9bcb1b39e"}, {"type": "file", "hashes": {"SHA-256": "890a0bfab48d0b93da8f7617b2e65621e8d7f8c93a854fa8596232d9bcb1b39e"}}),
        ({"type": "FileName", "value": "acmecorp.exe"}, {"type": "file", "name": "acmecorp.exe"}),
        # /entities endpoint
        ({"type": "ActorDomain", "value": "acme.com"}, {"type": "domain-name", "value": "acme.com"}),
        ({"type": "Handle", "value": "acme"}, {"type": "threat-actor", "name": "acme"}),
        ({"type": "ActorOtherWebsite", "value": "https://acme.com"}, {"type": "url", "value": "https://acme.com"}),
        ({"type": "BitcoinAddress", "value": "1HUu6K9sFvN1cGV"}, {"type": "cryptocurrency-wallet", "value": "1HUu6K9sFvN1cGV"}),
        ({"type": "OtherCryptoCurrencies", "value": "1HUu6K9sFvN1cGV"}, {"type": "cryptocurrency-wallet", "value": "1HUu6K9sFvN1cGV"}),
        ({"type": "EmailAddress", "value": "changeme@acme.com"}, {"type": "email-addr", "value": "changeme@acme.com"}),
        ({"type": "AIM", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "AIM"}),
        ({"type": "Discord", "value": "https://discord.gg/2xx2xx2"}, {"type": "user-account", "user_id": "2xx2xx2", "account_type": "Discord"}),
        ({"type": "Facebook", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Facebook"}),
        ({"type": "GitHub", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "GitHub"}),
        ({"type": "ICQ", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "ICQ"}),
        ({"type": "Instagram", "value": "instagram.com/changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Instagram"}),
        ({"type": "Jabber", "value": "changeme@jabber.de"}, {"type": "user-account", "user_id": "changeme@jabber.de", "account_type": "Jabber"}),
        ({"type": "LinkedIn", "value": "https://www.linkedin.com/in/iamweasel-58312b1a6/"}, {"type": "user-account", "user_id": "iamweasel-58312b1a6", "account_type": "LinkedIn"}),
        ({"type": "MSN", "value": "changeme@acme.com"}, {"type": "user-account", "user_id": "changeme@acme.com", "account_type": "MSN"}),
        ({"type": "MoiMir", "value": "my.mail.ru/mail/changeme/"}, {"type": "user-account", "user_id": "changeme", "account_type": "MoiMir"}),
        ({"type": "Odnoklassniki", "value": "ok.ru/profile/11111800259"}, {"type": "user-account", "user_id": "11111800259", "account_type": "Odnoklassniki"}),
        ({"type": "QQ", "value": "111159547"}, {"type": "user-account", "user_id": "111159547", "account_type": "QQ"}),
        ({"type": "Skype", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Skype"}),
        ({"type": "Telegram", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Telegram"}),
        ({"type": "TOX", "value": "2B41B398739E6BECE4E93EAFA0C665"}, {"type": "user-account", "user_id": "2B41B398739E6BECE4E93EAFA0C665", "account_type": "TOX"}),
        ({"type": "Twitter", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Twitter"}),
        ({"type": "VK", "value": "vk.com/changeme_2"}, {"type": "user-account", "user_id": "changeme_2", "account_type": "VK"}),
        ({"type": "WeChat", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "WeChat"}),
        ({"type": "Wickr", "value": "changeme"}, {"type": "user-account", "user_id": "changeme", "account_type": "Wickr"}),
        ({"type": "YahooIM", "value": "changeme@yahoo.com"}, {"type": "user-account", "user_id": "changeme@yahoo.com", "account_type": "YahooIM"}),
        ({"type": "PerfectMoneyID", "value": "U1111111"}, {"type": "user-account", "user_id": "U1111111", "account_type": "PerfectMoneyID"}),
        ({"type": "QiwiWallet", "value": "11110519386"}, {"type": "user-account", "user_id": "11110519386", "account_type": "QiwiWallet"}),
        ({"type": "WebMoneyID", "value": "111112935229"}, {"type": "user-account", "user_id": "111112935229", "account_type": "WebMoneyID"}),
        ({"type": "WebMoneyPurse", "value": "Z111144083730"}, {"type": "user-account", "user_id": "Z111144083730", "account_type": "WebMoneyPurse"}),
        ({"type": "YandexMoney", "value": "111113131482342"}, {"type": "user-account", "user_id": "111113131482342", "account_type": "YandexMoney"}),
        ({"type": "X", "value": "@FooB"}, {"type": "user-account", "user_id": "@FooB", "account_type": "X"}),
        ({"type": "X", "value": "https://x.com/foob"}, {"type": "user-account", "user_id": "foob", "account_type": "X"}),
        ({"type": "Signal", "value": "foo.11"}, {"type": "user-account", "user_id": "foo.11", "account_type": "Signal"}),
        ({"type": "Phone", "value": "79874172111"}, {"type": "user-account", "user_id": "79874172111", "account_type": "Phone"}),
        ({"type": "CveID", "value": "CVE-2024-23113"}, {"type": "vulnerability", "name": "CVE-2024-23113"}),
        # Invalid stuff
        ({"type": "Handle", "value": "a"}, None),
        ({"type": "MD5", "value": "invalidMd5"}, None),
        ({"type": "MaliciousDomain", "value": "https://invalid.domain"}, None)
))
def test_observable_mapper(source, expected_values):
    mapper = EntitiesMapper()
    sco = mapper.map(**source)
    if expected_values is None:
        assert sco is expected_values
    else:
        for key, value in expected_values.items():
            assert getattr(sco, key) == value


@pytest.mark.parametrize("report_type,source,expected_values", (
    (ReportType.FINTEL, {"id": "ab1", "type": "fintel", "sources": [
        {"type": "external", "title": "ACME corp news", "links": {"external": {"href": "https://acme.corp/123"}}, "index": "1"}]},
     {"source_name": "external - ACME corp news", "url": "https://acme.corp/123"}),
    (ReportType.FINTEL, {"id": "ab1", "type": "fintel", "sources": [
        {"type": "external", "source_type": "External Link", "title": "Titan Information Report",
         "links": {"external": {"href": "https://titan.intel471.com/report/inforep/487a8"}}, "index": "1"}]},
     {"source_name": "External Link - Titan Information Report",
      "url": "https://titan.intel471.com/report/inforep/487a8"}),
    (ReportType.INFOREP, {"id": "ab1", "type": "info_report", "sources": [
        {"type": "external", "source_type": "Forum Post", "title": "[SOURCE CODE] HexSec | Android RAT",
         "links": {"external": {"href": "https://titan.intel471.com/post_thread/9cacd56"}}, "index": "1"}]},
     {"source_name": "Forum Post - [SOURCE CODE] HexSec | Android RAT",
      "url": "https://titan.intel471.com/post_thread/9cacd56"}),
    (ReportType.INFOREP, {"id": "ab1", "type": "info_report", "sources": [
        {"type": "external", "source_type": "Forum Post", "title": "[SOURCE CODE] HexSec | Android RAT",
         "links": {"external": {"href": "https://titan.intel471.com/post_thread/9cacd56"}}, "index": "1"}]},
     {"source_name": "Forum Post - [SOURCE CODE] HexSec | Android RAT",
      "url": "https://titan.intel471.com/post_thread/9cacd56"}),
    (ReportType.BREACH_ALERT, {"id": "ab1", "type": "breach_alert", "sources": [
        {"type": "internal", "source_type": "Forum Thread", "title": "acmesystems",
         "links": {"verity_portal": {"href": "https://titan.intel471.com/post_thread/2984"}}, "index": "1"}]},
     {"source_name": "Forum Thread - acmesystems",
      "url": "https://titan.intel471.com/post_thread/2984"}),
    (ReportType.SPOTREP, {"id": "ab1", "type": "spot_report", "sources": [
        {"type": "internal", "title": "Forum thread",
         "links": {"verity_portal": {"href": "https://titan.intel471.com/post_thread/2984"}}, "index": "1"}]},
     {"source_name": "internal - Forum thread",
      "url": "https://titan.intel471.com/post_thread/2984"}),
))
def test_map_reports_external_references(report_type, source, expected_values):
    mapper = ReportMapper(STIXMapperSettings())
    external_refs = mapper._get_external_references(source)
    external_ref_0 = json.loads(external_refs[0].serialize())
    report_settings = ReportMapper.reports_settings.get(report_type)
    assert external_ref_0 == {"source_name": "Portal URL", "url": f"https://verity.intel471.com/{report_settings.portal_url_fragment}/ab1"}
    if expected_values:
        external_ref_1 = json.loads(external_refs[1].serialize())
        assert external_ref_1 == expected_values


@pytest.mark.parametrize("report_type,source", (
    (ReportType.FINTEL.value, {"id": "ab1", "type": "fintel", "entities": [{"type": "MalwareFamily", "value": "acme"}]}),
    (ReportType.FINTEL.value, {"id": "ab1", "type": "info_report", "entities": [{"type": "MalwareFamily", "value": "acme"}]}),
    (ReportType.BREACH_ALERT.value, {"id": "ab1", "type": "breach_alert", "entities": [{"type": "MalwareFamily", "value": "acme"}]}),
    (ReportType.SPOTREP.value, {"id": "ab1", "type": "spot_report", "entities": [{"type": "MalwareFamily", "value": "acme"}]}),
    (ReportType.MALWARE.value, {"id": "ab1", "type": "malware_report", "entities": [], "threat": {"id": "ac1", "type": "malware", "family": "acme"}}),
))
def test_map_reports_entities(report_type, source):
    mapper = ReportMapper(STIXMapperSettings())
    entities = mapper._get_entities(source)
    entity = json.loads(list(entities.get())[0].serialize())
    assert entity["type"] == "malware"
    assert entity["name"] == "acme"
    malware_families_names = mapper._get_malware_families_names(entities)
    assert malware_families_names == ["acme"]


@pytest.mark.parametrize("report_type,source", (
    (ReportType.FINTEL.value, {"id": "ab1", "type": "fintel", "victims": [
        {"name": "ACME corp", "links": [{"external": {"href": "https://acme.corp"}}]}]}),
    (ReportType.INFOREP.value, {"id": "ab1", "type": "info_report", "victims": [
        {"name": "ACME corp", "links": [{"external": {"href": "https://acme.corp"}}]}]}),
    (ReportType.BREACH_ALERT.value, {"id": "ab1", "type": "breach_alert", "victims": [{
            "country": "United States",
            "industries": [
                {
                    "industry": "Financial and investment consulting industry",
                    "sector": "Professional services and consulting sector"
                }
            ],
            "name": "ACME corp",
            "region": "North America",
            "revenue": "US $100 million",
            "links": [{
                "external": {
                    "href": "https://acme.corp"
                }
            }]
        }]}),
    (ReportType.SPOTREP.value, {"id": "ab1", "type": "spot_report", "victims": [
        {"name": "ACME corp", "links": [{"external": {"href": "https://acme.corp"}}]}]}),
))
def test_map_reports_victims(report_type, source):
    mapper = ReportMapper(STIXMapperSettings())
    victims = list(mapper._get_victims(source).get())
    assert victims[0].type == "identity"
    assert victims[0].identity_class == "organization"
    assert victims[0].name == "ACME corp"
    assert victims[0].contact_information == "https://acme.corp"


def test_indicator_pattern_url_quote():
    mapper = IndicatorsMapper(STIXMapperSettings())
    result = mapper.map({"count": "1", "indicators": [{
        "activity": {"first_seen_ts": "2018-08-07T18:33:00Z", "last_seen_ts": "2018-08-07T18:33:00Z"},
        "data": {
            "url": "https://foo.bar.com/pixel.png'ogi,TO"
            },
        "pattern": "[url:value = 'https://foo.bar.com/pixel.png'ogi,TO']",
        "pattern_type": "stix",
        "pattern_version": "2.1",
        "type": "url",
        "threat": {"data": {"malware_family": {"name": "foo"}}},
        "expiration_ts": "2023-01-26T01:03:54Z",
        "kill_chain_phases": [
            {
                "kill_chain_name": "mitre-attack",
                "phase_name": "command_and_control"
            }
        ],
        "confidence": 50,
        "description": "foo",
        "classification": {
            "girs": [
                {
                    "name": "Banking trojan malware",
                    "path": "1.1.4"
                }
            ]
        }
    }]})
    assert result is None


def test_reports_stream_not_mappable():
    """
    Only allow STIX mappings on type-specific endpoints, such as /reports/fintel/stream, etc.
    as /reports/stream's data is too limited.
    """
    mapper = StixMapper(STIXMapperSettings())
    with pytest.raises(StixMapperNotFound):
        mapper.map(
            {
                "count": 1740,
                "info_report_count": 0,
                "fintel_report_count": 1740,
                "breach_alert_count": 0,
                "spot_report_count": 0,
                "malware_report_count": 0,
                "vulnerability_report_count": 0,
                "geopol_report_count": 0,
                "cursor_next": "MjhiMWQ2ZGItMzJkNi00NDFkLWMyOTItOTk0YjI0YmRkMjE3OjE3NjI5NDAwNTI3MTA6cmVwb3J0LS1lNzFkMzg3Zi0zMjVlLTViZmMtYTQzZC0xNDM4NzZjNmNmYzA",
                "reports": [
                    {
                        "id": "report--e71d387f-325e-5bfc-a43d-143876c6cfc0",
                        "type": "fintel",
                        "sub_type": "actor_profile",
                        "title": "Foo",
                    }
                ]
            }
        )


def test_map_invalid_indicator():
    mapper = IndicatorsMapper(STIXMapperSettings())
    result = mapper.map({
        "id": "abc",
        "type": "domain",
        "data": {"domain": "http://foobar.com"},
        "confidence": 50,
        "pattern": "[domain-name:value = 'http://foobar.com']",
        "pattern_type": "stix",
        "pattern_version": "2.1",
        "threat": {"data": {"malware_family": {"name": "Foo"}}},
        "activity": {"first_seen_ts": "2026-01-23T15:24:54.356273Z"}
    })
    print(result.serialize())


# coding: utf-8

"""
    Verity471 Unified API

      - creds: 1.0.1
  - indicators: 1.0.0
  - malware: 1.0.0
  - reports: 1.0.0
  - sources: 1.0.2
  - actors: 1.0.1
  - watchers: 1.0.0
  - observables: 1.0.0
  - entities: 1.0.0
  - girs: 1.0.0


    The version of the OpenAPI document: 1.1.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from verity471.models.industries import Industries
from verity471.models.links import Links
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

if TYPE_CHECKING:
    # Only for type checkers; does not run at runtime.
    from verity471.verity_stix import STIXMapperSettings


class ReportsVictimResponse(BaseModel):
    """
    ReportsVictimResponse
    """ # noqa: E501
    country: Optional[StrictStr] = Field(default=None, description="Country of the victim")
    industries: Optional[List[Industries]] = Field(default=None, description="List of Industries associated with the victim")
    links: Optional[List[Links]] = Field(default=None, description="List of URLs associated with the victim")
    name: StrictStr = Field(description="Name of the victim")
    region: Optional[StrictStr] = Field(default=None, description="Region information of the victim")
    revenue: Optional[StrictStr] = Field(default=None, description="Revenue information of the victim")
    __properties: ClassVar[List[str]] = ["country", "industries", "links", "name", "region", "revenue"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_stix(self, settings: Optional["STIXMapperSettings"] = None):
        try:
            from verity471.verity_stix.mappers.common import StixMapper
        except ImportError as e:
            raise ImportError(
                "STIX support is an optional feature. "
                "Install with: pip install 'verity471[stix]'"
            ) from e
        stix_mapper = StixMapper(settings)
        return stix_mapper.map(self.to_dict())

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ReportsVictimResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in industries (list)
        _items = []
        if self.industries:
            for _item_industries in self.industries:
                if _item_industries:
                    _items.append(_item_industries.to_dict())
            _dict['industries'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item_links in self.links:
                if _item_links:
                    _items.append(_item_links.to_dict())
            _dict['links'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ReportsVictimResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "country": obj.get("country"),
            "industries": [Industries.from_dict(_item) for _item in obj["industries"]] if obj.get("industries") is not None else None,
            "links": [Links.from_dict(_item) for _item in obj["links"]] if obj.get("links") is not None else None,
            "name": obj.get("name"),
            "region": obj.get("region"),
            "revenue": obj.get("revenue")
        })
        return _obj





import yaml
import pycti
from stix2 import Bundle, Vulnerability, ExternalReference, Software, Relationship

from .common import BaseMapper, StixMapper
from .. import author_identity, StixObjects
from ..constants import MARKING, PLATFORM_VERITY471, X_OPENCTI_CREATED_BY, X_OPENCTI_LABELS


@StixMapper.register(
    "vuln_reports",
    lambda x: isinstance(x.get("reports"), list)
    and (x["reports"][0].get("id") or "").startswith("vulnerability--")
    and "fintel_report_count" not in x,
)
@StixMapper.register(
    "vuln_report",
    lambda x: isinstance(x, dict) and (x.get("id") or "").startswith("vulnerability--"),
)
class CveMapper(BaseMapper):
    def map(self, source: dict) -> Bundle:
        container = StixObjects()
        items = (
            source.get("reports") or []
            if "count" in source
            else [source]
        )
        for item in items:
            id_ = item["id"]
            name = item["name"]
            summary = item["body"]
            underground_activity_summary = item["underground_activity_summary_html"]
            counter_measures = item["counter_measures_html"]
            extras = yaml.dump(
                {
                    k: v
                    for k, v in item.items()
                    if k
                    in (
                        "patch_status",
                        "interest_level",
                        "activity_location",
                        "exploit_status",
                    )
                }
            )
            description = f"{summary}\n\n{underground_activity_summary}\n\n{counter_measures}\n\n### Properties\n\n```yaml\n{extras}```"

            external_references = []
            # New links structure
            links = self.map_links(item.get("links"))
            for link in links:
                external_references.append(ExternalReference(source_name=link.type, url=link.href))

            # Old style links, to be changed in MER-7243
            for link_type, key in (
                ("Verity URL", "sources"),
                ("PoC", "poc_links"),
                ("Patch", "patch_links"),
                ("Counter measures", "counter_measure_links")
            ):
                for link in item.get(key) or []:
                    external_reference = ExternalReference(
                        source_name=f"[{link_type}] {link['title']}", url=link["url"]
                    )
                    external_references.append(external_reference)

            custom_properties = {"x_intel471_com_uid": id_}
            cvss_item = next((i for i in item.get("cvss") if i.get('version') == '3.1'), None)
            cvss3_score = cvss_item.get('score') if cvss_item and cvss_item.get('score', 0) > 0 else None
            if cvss3_score:
                custom_properties["x_opencti_base_score"] = cvss3_score
            labels = self.get_girs_labels((item.get("classification") or {}).get("girs") or [])
            labels.append(PLATFORM_VERITY471)
            vulnerability = Vulnerability(
                id=pycti.Vulnerability.generate_id(name),
                name=name,
                description=description,
                created_by_ref=author_identity,
                external_references=external_references,
                object_marking_refs=[MARKING],
                custom_properties=custom_properties,
                labels=labels
            )
            software = Software(
                name=item["product_name"],
                vendor=item["vendor_name"],
                custom_properties={
                    X_OPENCTI_CREATED_BY: author_identity.id,
                    X_OPENCTI_LABELS: [PLATFORM_VERITY471]
                    })
            rel = Relationship(
                id=pycti.StixCoreRelationship.generate_id(
                    relationship_type="has",
                    source_ref=software.id,
                    target_ref=vulnerability.id),
                source_ref=software,
                relationship_type= "has",
                target_ref=vulnerability,
                object_marking_refs=[MARKING],
                labels=[PLATFORM_VERITY471],
                created_by_ref=author_identity)
            container.extend([vulnerability, software, rel, author_identity, MARKING])
        if container:
            bundle = Bundle(*container.get(), allow_custom=True)
            return bundle
